#!/usr/bin/env python
# coding: utf-8

import json
import os
import sys
import traceback

import requests
from requests.auth import HTTPBasicAuth
from requests.structures import CaseInsensitiveDict


class _MissingRequiredParameter(Exception):

    def __init__(self,
                 arg_name,
                 env_var_name,
                 object_type,
                 message='Missing a required Parameter'):
        self.arg_name = arg_name
        self.env_var_name = env_var_name
        self.message = message
        self.object_type = object_type
        super().__init__(self.message)

    def __str__(self):
        return f'Missing a valid {self.arg_name.replace("_", " ")}! Either ' \
        f'pass it to the {self.object_type} as: `{self.arg_name}=...`, or ' \
        f'set it as an environment variable: `{self.env_var_name}`.'


class Gotify:

    def __init__(self,
                 host_address=None,
                 fixed_token=None,
                 fixed_priority=None):
        self.host_address = host_address
        self.fixed_token = fixed_token
        self.fixed_priority = fixed_priority

    @staticmethod
    def _headers():
        headers = CaseInsensitiveDict()
        headers['Content-type'] = 'application/json'
        return headers

    def _get_host_address(self):
        host_address = self.host_address
        if not host_address and not os.getenv('GOTIFY_HOST_ADDRESS'):
            raise _MissingRequiredParameter('host_address',
                                            'GOTIFY_HOST_ADDRESS',
                                            'class instance')
        else:
            if not host_address:
                host_address = os.getenv('GOTIFY_HOST_ADDRESS')
        if 'http' not in host_address:
            raise TypeError(
                'Missing a valid scheme in the host address (e.g., `http://`)!'
            )
        host_address = host_address.rstrip('/')
        return host_address

    def create_app(self, admin_username, admin_password, app_name, desc=None):
        host_address = self._get_host_address()
        url = f'{host_address}/application'
        data = json.dumps({'name': app_name, 'description': desc})
        auth = HTTPBasicAuth(admin_username, admin_password)
        resp = requests.post(url,
                             headers=self._headers(),
                             data=data,
                             auth=auth)
        return resp.json()

    def push(self, title, message, token=None, priority=2):

        if not token:
            token = self.fixed_token
            if not token:
                token = os.getenv('GOTIFY_APP_TOKEN')
        host_address = self.host_address

        if self.fixed_priority:
            priority = self.fixed_priority

        data = {'title': title, 'message': message, 'priority': priority}

        if not token and not os.getenv('GOTIFY_APP_TOKEN'):
            raise _MissingRequiredParameter('token', 'GOTIFY_APP_TOKEN',
                                            'method')

        host_address = self._get_host_address()

        url = f'{host_address}/message?token={token}'
        resp = requests.post(url,
                             headers=self._headers(),
                             data=json.dumps(data))

        try:
            return resp.json()
        except json.decoder.JSONDecodeError:
            traceback.print_exception(*sys.exc_info())
        except requests.exceptions.ConnectionError:
            traceback.print_exception(*sys.exc_info())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gotipy']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'gotipy',
    'version': '0.2.2',
    'description': 'A simple Python wrapper for Gotify',
    'long_description': "## Gotipy\n\nA simple Python wrapper for [Gotify](https://github.com/gotify/server)\n\n## Requirements\n- [python>=3.6](https://www.python.org/downloads/)\n- [gotify/server](https://gotify.net/docs/install)\n\n\n## Installation\n\n```sh\npip install gotipy\n```\n\n## Usage\n\n```py\nfrom gotipy import Gotify\n\ng = Gotify(host_address='https://yourdomain.com')\n```\n\n### Create a new application\n\n```py\ng.create_app(admin_username='admin',\n             admin_password='admin',\n             app_name='demo',\n             desc='My first app!')\n# {\n#     'id': 1,\n#     'token': 'xxxxxxxxxx',\n#     'name': 'demo',\n#     'description': 'My first app!',\n#     'internal': False,\n#     'image': 'static/defaultapp.png'\n# }\n```\n\n### Push messages to an application\n\n```py\ng.push(title='My message',\n       message='Hello, World!',\n       priority=9,\n       token='xxxxxxxxxx')\n# {\n#     'id': 1,\n#     'appid': 1,\n#     'message': 'Hello, World!',\n#     'title': 'My message',\n#     'priority': 9,\n#     'date': '2022-05-16T05:25:29.367216435Z'\n# }\n```\n\n\n### Notes:\n\n- You can set `GOTIFY_HOST_ADDRESS` as an envionmnt variable instead of passing `host_address` to the class instance.\n- If you want to use the same app in all your `push` calls, you can omit `token` from the method, and pass `fixed_token=...` to the class instance or set `GOTIFY_APP_TOKEN` as an envionmnt variable instead.\n- If you want to set a fixed priority level for all your `push` calls, pass `fixed_priority` to the class instance.\n",
    'author': 'Mohammad Alyetama',
    'author_email': 'malyetama@pm.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
